"use strict";
/*
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateDataset = void 0;
const util_1 = require("util");
const cli_1 = require("@zowe/cli");
const imperative_1 = require("@zowe/imperative");
const JobUtils_1 = require("./JobUtils");
const CoreUtils_1 = require("./CoreUtils");
const AbstractTemplatedJCL_1 = require("./AbstractTemplatedJCL");
class CreateDataset extends AbstractTemplatedJCL_1.AbstractTemplatedJCL {
    constructor() {
        super(...arguments);
        this.DEFAULT_TEMPLATE = "//CREATE EXEC PGM=IEFBR14\n" +
            "//{{createDD}}";
    }
    /**
     * Create a data set
     * @param connection                 - zos-node-accessor connection
     * @param  cmdType               - The type of data set we are going to create
     * @param dataSetName                          - the name of the data set to create
     * @param  [options={}] - additional options for the creation of the data set
     */
    static create(connection, cmdType, dataSetName, jobCardFile, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let validCmdType = true;
            // Removes undefined properties
            let tempOptions = !(0, util_1.isNullOrUndefined)(options) ? JSON.parse(JSON.stringify(options)) : {};
            const secondarySpecified = !(0, util_1.isNullOrUndefined)(tempOptions.secondary);
            // Required
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(cmdType, cli_1.ZosFilesMessages.missingDatasetType.message);
            // Required
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(dataSetName, cli_1.ZosFilesMessages.missingDatasetName.message);
            switch (cmdType) {
                case 3 /* CreateDataSetTypeEnum.DATA_SET_PARTITIONED */:
                    tempOptions = Object.assign(Object.assign({}, cli_1.CreateDefaults.DATA_SET.PARTITIONED), tempOptions);
                    break;
                case 4 /* CreateDataSetTypeEnum.DATA_SET_SEQUENTIAL */:
                    tempOptions = Object.assign(Object.assign({}, cli_1.CreateDefaults.DATA_SET.SEQUENTIAL), tempOptions);
                    break;
                case 0 /* CreateDataSetTypeEnum.DATA_SET_BINARY */:
                    tempOptions = Object.assign(Object.assign({}, cli_1.CreateDefaults.DATA_SET.BINARY), tempOptions);
                    break;
                case 1 /* CreateDataSetTypeEnum.DATA_SET_C */:
                    tempOptions = Object.assign(Object.assign({}, cli_1.CreateDefaults.DATA_SET.C), tempOptions);
                    break;
                case 2 /* CreateDataSetTypeEnum.DATA_SET_CLASSIC */:
                    tempOptions = Object.assign(Object.assign({}, cli_1.CreateDefaults.DATA_SET.CLASSIC), tempOptions);
                    break;
                default:
                    validCmdType = false;
                    break;
            }
            if (!validCmdType) {
                throw new imperative_1.ImperativeError({ msg: cli_1.ZosFilesMessages.unsupportedDatasetType.message });
            }
            else {
                // Handle the size option
                if (!(0, util_1.isNullOrUndefined)(tempOptions.size)) {
                    const tAlcunit = tempOptions.size.toString().match(/[a-zA-Z]+/g);
                    if (!(0, util_1.isNullOrUndefined)(tAlcunit)) {
                        tempOptions.alcunit = tAlcunit.join("").toUpperCase();
                    }
                    const tPrimary = tempOptions.size.toString().match(/[0-9]+/g);
                    if (!(0, util_1.isNullOrUndefined)(tPrimary)) {
                        tempOptions.primary = +(tPrimary.join(""));
                        if (!secondarySpecified) {
                            const TEN_PERCENT = 0.10;
                            tempOptions.secondary = Math.round(tempOptions.primary * TEN_PERCENT);
                        }
                    }
                    delete tempOptions.size;
                }
                let response = "";
                // Handle the print attributes option
                if (!(0, util_1.isNullOrUndefined)(tempOptions.printAttributes)) {
                    if (tempOptions.printAttributes) {
                        delete tempOptions.printAttributes;
                        response = imperative_1.TextUtils.prettyJson(tempOptions);
                    }
                    else {
                        delete tempOptions.printAttributes;
                    }
                }
                response = yield new CreateDataset().createViaFTP(connection, dataSetName, tempOptions, jobCardFile);
                if (response.indexOf("RC=0000") >= 0) {
                    response += "\n" + cli_1.ZosFilesMessages.dataSetCreatedSuccessfully.message;
                }
                else {
                    response += "\nFailed to create data set";
                }
                return {
                    success: true,
                    commandResponse: response
                };
            }
        });
    }
    createViaFTP(connection, dataSetName, options, jobCardFile, overrideTemplateFile) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug("Building jcl to create data set %s", dataSetName);
            let createDD = "NEWDATA DD ";
            const createDDValues = ["DSN=" + dataSetName.toUpperCase(), "DISP=(NEW,CATLG)"];
            const maxDDLineLength = 65;
            let currentLineLength = createDD.length;
            const continueLine = "//   ";
            // todo: make sure all options are used here
            if (options.alcunit) {
                let spaceParams = "SPACE=(";
                spaceParams += options.alcunit.toUpperCase() + ",(";
                spaceParams += options.primary;
                if (options.secondary != null) {
                    spaceParams += "," + options.secondary;
                }
                spaceParams += "))";
                createDDValues.push(spaceParams);
                if (options.volser != null) {
                    createDDValues.push("VOLSER=" + options.volser.toUpperCase());
                }
            }
            if (options.lrecl != null) {
                createDDValues.push("LRECL=" + options.lrecl);
            }
            if (options.recfm != null) {
                createDDValues.push("RECFM=" + options.recfm.toUpperCase());
            }
            if (options.blksize != null) {
                createDDValues.push("BLKSIZE=" + options.blksize);
            }
            if (options.unit != null) {
                createDDValues.push("UNIT=" + options.unit.toUpperCase());
            }
            if (options.dsorg != null) {
                createDDValues.push("DSORG=" + options.dsorg.toUpperCase());
            }
            /**
             * Build the DD definition for the job by looping through the values we've built
             */
            for (let i = 0; i < createDDValues.length; i++) {
                const value = createDDValues[i];
                if (value.length + currentLineLength > maxDDLineLength) {
                    createDD += "\n" + continueLine;
                    currentLineLength = continueLine.length;
                }
                createDD += value;
                if (i < createDDValues.length - 1) {
                    // if this isn't the last value, add a comma to continue the line
                    createDD += ",";
                    currentLineLength += 1;
                }
                currentLineLength += value.length;
            }
            const jcl = this.getJcl(jobCardFile, { createDD }, overrideTemplateFile);
            this.log.debug("Creating data set %s via JCL:\n %s", dataSetName, jcl);
            const jobid = yield connection.submitJCL(jcl);
            const jobWait = 500;
            yield CoreUtils_1.CoreUtils.sleep(jobWait); // have to wait for the job to show up in the list
            const jobDetails = yield JobUtils_1.JobUtils.findJobByID(connection, jobid);
            return "Job details: " + JSON.stringify(jobDetails, null, 2);
        });
    }
}
exports.CreateDataset = CreateDataset;
//# sourceMappingURL=CreateDataset.js.map