"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startRegion = exports.startTransaction = exports.startProgram = void 0;
const imperative_1 = require("@zowe/imperative");
const rest_1 = require("../../rest");
const constants_1 = require("../../constants");
const CommonUtils_1 = require("../CommonUtils");
const ImsSessionUtils_1 = require("../../../cli/ImsSessionUtils");
/**
 * Start program in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IUpdateProgramParms} parms - parameters for start program(s)
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS program name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function startProgram(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toNotBeNullOrUndefined(parms.name, "IMS program name is required");
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.name[0], "IMS program name", "IMS program name is required");
        imperative_1.Logger.getAppLogger().debug("Attempting to start program(s) with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.PROGRAM;
        resource = resource + CommonUtils_1.getQueryFromParms(parms, { start: "SCHD" });
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.startProgram = startProgram;
/**
 * Start transaction in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IUpdateTransactionParms} parms - parameters for starting transaction(s)
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS transaction name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function startTransaction(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toNotBeNullOrUndefined(parms.name, "IMS transaction name is required");
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.name[0], "IMS transaction name", "IMS transaction name is required");
        imperative_1.Logger.getAppLogger().debug("Attempting to start transactions(s) with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.TRANSACTION;
        resource = resource + CommonUtils_1.getQueryFromParms(parms, { start: "SCHD" });
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.startTransaction = startTransaction;
/**
 * Start region in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IStartRegionParms} parms - parameters for starting a region, see interface for more details
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS member name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function startRegion(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.memberName, "IMS member name", "IMS member name is required");
        imperative_1.Logger.getAppLogger().debug("Attempting to start a region with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.REGION + "/" + constants_1.ImsConstants.START;
        resource = resource + CommonUtils_1.getQueryFromParms(parms, {}, { memberName: "member_name" });
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.startRegion = startRegion;
//# sourceMappingURL=Start.js.map