import { ICommandHandler, IHandlerParameters } from "@zowe/imperative";
import { IIMSApiResponse } from "../api/doc/IIMSApiResponse";
import { ImsSession } from "../api/rest";
/**
 * This class is used by the various ims handlers as the base class for their implementation.
 * All handlers should extend this class whenever possible
 */
export declare abstract class ImsBaseHandler implements ICommandHandler {
    /**
     * Local reference to commandParameters for convenience
     */
    protected params: IHandlerParameters;
    /**
     * This will grab the ims profile and create a session before calling the subclass
     * {@link ImsBaseHandler#processWithSession} method.
     *
     * @param {IHandlerParameters} commandParameters Command parameters sent by imperative.
     *
     * @returns {Promise<void>}
     */
    process(commandParameters: IHandlerParameters): Promise<void>;
    /**
     * This is called by the {@link ImsBaseHandler#process} after it creates a session. Should
     * be used so that every class does not have to instantiate the session object.
     *
     * @param {IHandlerParameters} commandParameters Command parameters sent to the handler.
     * @param {ImsSession} session The session object generated from the ims profile.
     *
     * @returns {Promise<IIMSApiResponse>} The response from the underlying ims api call.
     */
    abstract processWithSession(commandParameters: IHandlerParameters, session: ImsSession): Promise<IIMSApiResponse>;
    /**
     * Check return codes for messages in case we don't receive a bad HTTP status code but
     * do receive a bad RC in one of the messages returned by the API
     */
    protected checkReturnCode(response: IIMSApiResponse): void;
}
